@@ -1,6 +1,6 @@
 #! /bin/false
 
-# Copyright (C) 2021-2025 Guido Flohr <guido.flohr@cantanea.com>,
+# Copyright (C) 2021 Guido Flohr <guido.flohr@cantanea.com>,
 # all rights reserved.
 
 # This program is free software. It comes without any warranty, to
@@ -38,11 +38,11 @@ sub new {
 	my $info = $args{info} || sub {};
 	my $tt = $args{tt} || Chess::Plisco::Engine::TranspositionTable->new(16);
 	my $tree = Chess::Plisco::Engine::Tree->new(
-		position => $position->copy,
-		tt => $tt,
-		watcher => $watcher,
-		info => $info,
-		signatures => \@signatures,
+		$position->copy,
+		$tt,
+		$watcher,
+		$info,
+		\@signatures,
 	);
 
 	my $tc = Chess::Plisco::Engine::TimeControl->new($tree, %args);
